#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_SET_LED    0
#define VENDOR_RQ_GET_SWITCH 1

#define port0  0
#define port1  1
#define port2  2
#define port3  3
#define port4  4
#define port5  5

#define GET_FORWARD 10
#define GET_BACKWARD 20
/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

uint8_t in_c(uint8_t pin)
{
    // read logic level of port-C's pin
    DDRC &= ~(1<<pin);
    return (PINC >>pin) & 1;
} 

uint8_t in_b(uint8_t pin)
{
    // read logic level of port-C's pin
    DDRB &= ~(1<<pin);
    return (PINB >>pin) & 1;
}

void out_c(uint8_t pin, uint8_t val)
{
    // write logic level to port-C's pin
    DDRC |= 1<<pin;
    if (val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
	//int portnum;
    usbRequest_t *rq = (void *)data;
    static uint8_t dataBuffer[1];  /* must stay valid when usbFunctionSetup returns */

    if(rq->bRequest==port0)
	{
        dataBuffer[0] = in_c(0);
        usbMsgPtr = dataBuffer;
        return 1;
	}
	else if(rq->bRequest==port1)
	{
        dataBuffer[0] = in_c(1);
        usbMsgPtr = dataBuffer;
        return 1;
	}
	else if(rq->bRequest==port2)
	{
        dataBuffer[0] = in_c(2);
        usbMsgPtr = dataBuffer;
        return 1;
	}
	else if(rq->bRequest==port3)
	{
        dataBuffer[0] = in_c(3);
        usbMsgPtr = dataBuffer;
        
        return 1;
	}
	else if(rq->bRequest==port4)
	{
        dataBuffer[0] = in_c(4);
        usbMsgPtr = dataBuffer;
        return 1;
	}
	else if(rq->bRequest==port5)
	{
        dataBuffer[0] = in_c(5);
        usbMsgPtr = dataBuffer;
        return 1;
	}
	else if(rq->bRequest==GET_FORWARD)
	{
		dataBuffer[0] = in_b(4);	//Read forward from PB13
        usbMsgPtr = dataBuffer;
        return 1;
	}
	else if(rq->bRequest==GET_BACKWARD)
	{
		dataBuffer[0] = in_b(2);	//Read BREAK from PB12
        usbMsgPtr = dataBuffer;
        return 1;
	}
	return 0;
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    //DDRC  = (0<<PC0)|(0<<PC1)|(0<<PC2)|(1<<PC0)|(1<<PC1)|(1<<PC2); // PC0..PC2 = Output, PC3..PC5 = Input
    //PORTC = (1 << PC3);                 // Enable pull-up at PC3
	DDRC  = 0b11000000;		//Set INPUT = 0, to PC0 ... PC4
	PORTC = 0b00000000;
	
	DDRB  = 0b11000000;		//Set INPUT =0, to PC3 and PC5
	PORTB = 0b00000000;
	
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){                /* main event loop */
        usbPoll();
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
